/*
 * IoCompletionInit.c: Io command completion object.
 * Copyright (c) 2004 Jonas Eschenburg <indyjo@gmx.de>
 *
 * Portions
 * Copyright (c) 2004 Kentaro A. Kurahone.  All Rights Reserved.
 */

#include "IoState.h"
#include "IoObject.h"

void IoCompletionInit(IoState *self, IoObject *context)
{
  char *s = "String beginsWith := method(s, s == slice(0, s size))\n"
"String endsWith := method(s, s == slice(size - s size, size))\n"
"\n"
"Completion := Object clone do(\n"
"  openers := Map clone atPut(\")\",\"(\") atPut(\"}\",\"{\") atPut(\"]\",\"[\")\n"
"  lastCommandOf := method(str,\n"
"    inString := nil\n"
"    inComment := nil\n"
"    braces := list\n"
"    stack := list(0)\n"
"    for(i, 0, str size - 1,\n"
"      c := str slice(i,i+1)\n"
"      if (inComment and c==\"\\n\",\n"
"        inComment = nil; continue)\n"
"      if (inString and c==\"\\\"\" and str slice(i-1,i)  != \"\\\\\", \n"
"        inString = nil; continue)\n"
"      if (c == \"#\" or c==\"/\" and i != (str size - 1) and str slice(i+1,i+2) == \"/\",\n"
"        inComment = 1; continue)\n"
"      if (c == \"(\" or c==\"{\" or c==\"[\",\n"
"        stack push(i+1)\n"
"        braces push(c))\n"
"      if (c == \")\" or c==\"}\" or c==\"]\",\n"
"        if (braces pop != openers at(c), return nil, stack pop))\n"
"      if (c == \";\" or c==\"\\n\" or c==\",\",\n"
"        stack pop\n"
"        stack push(i + 1))\n"
"    )\n"
"    if (inString or inComment, return nil)\n"
"    command := str slice(stack pop)\n"
"  )\n"
"    \n"
"  complete := method(str, context,\n"
"    command := lastCommandOf(str asMutable strip(\" \"))\n"
"    if (command isNil, return list)\n"
"    \n"
"    try(\n"
"      msg := message(thisContext) setAttachedMessage( command asMessage )\n"
"    ) catch (\n"
"      return list\n"
"    )\n"
"    prefix := \"\"\n"
"\n"
"    if (command endsWithSeq(\" \") not,\n"
"      while (msg attachedMessage attachedMessage, msg = msg attachedMessage)\n"
"      prefix = msg attachedMessage name\n"
"      msg setAttachedMessage(nil)\n"
"    )\n"
"\n"
"    res := softEval(context, msg)\n"
"    if (res,\n"
"      slots := allSlotNamesOf(res) select(i,v, v beginsWithSeq(prefix))\n"
"      slots = slots map (i,v, v slice(prefix size)) sort\n"
"    ,\n"
"      list)\n"
"  )\n"
"  \n"
"  // Tries to evaluate msg in context without doing anything that might\n"
"  // cause side effects.\n"
"  // If successful, returns the result\n"
"  // Else, returns nil\n"
"  softEval := block(context, msg,\n"
"    if (msg name == \"\" and msg arguments size == 0, return nil)\n"
"    obj := context\n"
"    while (msg,\n"
"      if (msg name asNumber) then (\n"
"        obj = msg name asNumber\n"
"      ) elseif (msg name beginsWithSeq(\"\\\"\") and msg name endsWithSeq(\"\\\"\")) then (\n"
"        obj = msg name asMutable unescape\n"
"      ) else (\n"
"        if (theslot := context getSlot( msg name ) ) then (\n"
"          if (msg name == \"thisContext\",\n"
"            obj = context\n"
"            ,\n"
"            if (theslot type == \"CFunction\",\n"
"              return nil\n"
"              ,\n"
"              if (theslot type == \"Block\",\n"
"                return nil\n"
"                ,\n"
"                obj = theslot\n"
"              )\n"
"            )\n"
"          )\n"
"        ) else (\n"
"          return nil\n"
"        )\n"
"      )\n"
"      msg = msg attachedMessage\n"
"    )\n"
"    return obj\n"
"  )\n"
"  \n"
"  allSlotNamesOf := block(obj,\n"
"    if (obj != Object,\n"
"      names := List clone\n"
"      obj protos foreach(i, v, names appendSeq(allSlotNamesOf(v)))\n"
"    ,\n"
"      names := list)\n"
"    obj slotNames foreach(i,v, if( names contains(v) not, names append(v)))\n"
"    return names\n"
"  )\n"
"\n"
"  // IoFE doComplete code.\n"
"  //  * Prints if there are multiple results, returns nil\n"
"  //  * If there is one result, returns string containing result.\n"
"  //  * If there are no results, returns nil\n"
"  doComplete := block(msg,\n"
"    result := complete(msg, Lobby)\n"
"\n"
"    if (result size == 0, return nil)\n"
"    if (result size == 1, return result at(0))\n"
"\n"
"    \"\\n\" print\n"
"    for (i, 0, result size - 1,\n"
"      result at(i) print\n"
"      \" \" print\n"
"    )\n"
"    \"\\n\" print\n"
"    return nil\n"
"  )\n"
"\n"
")\n"
;
  IoObject_rawDoString_label_(context, SIOSYMBOL(s), SIOSYMBOL("Completion"));
}

