/*
 * IoEditLine.c: Editline control object.
 * Copyright (c) 2004 Kentaro A. Kurahone.  All Rights Reserved.
 */

#include <stdio.h>
#include <histedit.h>

#include "IoState.h"
#include "IoNumber.h"
#include "IoSeq.h"
#include "IoEditLine.h"

/* Static functions. */
static IoObject* IoEditLine_getHistSize(IoObject *self, IoObject *locals, IoMessage *m);
static IoObject* IoEditLine_setHistSize(IoObject *self, IoObject *locals, IoMessage *m);
static IoObject* IoEditLine_getKeyStyle(IoObject *self, IoObject *locals, IoMessage *m);
static IoObject* IoEditLine_setKeyStyle(IoObject *self, IoObject *locals, IoMessage *m);

/* External variables. */
extern History *histCtxt;
extern EditLine *editCtxt;
extern int histSize;

/* Static globals. */
static int isEmacs = 1;

void newEditLineObject(IoState *state, IoObject *lobby)
{
  IoObject *ctrlObj;
  IoObject *self;

  /* Insert the object into the lobby. */
  ctrlObj = IoState_doCString_(state, "IoFE EditLine := Object clone");
  self = ctrlObj;

  /* HACK: el_get for EL_EDITOR seems to be busted.
   * Set the default edit key bindings here and use a global to keep track.
   */
  el_set(editCtxt, EL_EDITOR, "emacs");

  /* Insert the object slots. */
  IoObject_addMethod_(ctrlObj, IOSYMBOL("setHistSize"), IoEditLine_setHistSize);
  IoObject_addMethod_(ctrlObj, IOSYMBOL("getHistSize"), IoEditLine_getHistSize);
  IoObject_addMethod_(ctrlObj, IOSYMBOL("setKeyStyle"), IoEditLine_setKeyStyle);
  IoObject_addMethod_(ctrlObj, IOSYMBOL("getKeyStyle"), IoEditLine_getKeyStyle);
}


static IoObject* IoEditLine_getHistSize(IoObject *self, IoObject *locals, IoMessage *m)
{
#if 0
  int size;
  HistEvent ev;

  history(histCtxt, &ev, H_GETSIZE);
  size = ev.num;
#endif
  /* HACK: libedit does not provide an api for querying max history size,
   * so use a global instead.
   */

  return IONUMBER(histSize);
}

static IoObject* IoEditLine_setHistSize(IoObject *self, IoObject *locals, IoMessage *m)
{
  HistEvent ev;
  int size;

  IoMessage_assertArgCount_receiver_(m, 1, self);
  size = IoMessage_locals_intArgAt_(m, locals, 0);

  histSize = size;
  history(histCtxt, &ev, H_SETSIZE, size);

  return IONUMBER(histSize);
}

static IoObject* IoEditLine_getKeyStyle(IoObject *self, IoObject *locals, IoMessage *m)
{
  return (isEmacs)
    ? IoSeq_newWithCString_(IOSTATE, "emacs")
    : IoSeq_newWithCString_(IOSTATE, "vi");
}

static IoObject* IoEditLine_setKeyStyle(IoObject *self, IoObject *locals, IoMessage *m)
{
  IoObject *style;

  IoMessage_assertArgCount_receiver_(m, 1, self);
  style = IoMessage_locals_seqArgAt_(m, locals, 0);

  if (strcmp(IoSeq_asCString(style), "emacs") == 0) {
    el_set(editCtxt, EL_EDITOR, "emacs");
    isEmacs = 1;
  } else if (strcmp(IoSeq_asCString(style), "vi") == 0) {
    el_set(editCtxt, EL_EDITOR, "vi");
    isEmacs = 0;
  } else {
     IoState_error_(IOSTATE, m,
      "EditLine.invalidKeyStyle: keyStyle must be \"emacs\" or \"vi\"");
  }

  return style;
}
